@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,19 +57,22 @@ import org.osgi.util.tracker.ServiceTracker;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import com.openexchange.charset.CollectionCharsetProvider;
 import com.openexchange.charset.ModifyCharsetExtendedProvider;
+import com.openexchange.charset.ModifyCharsetStandardProvider;
 
 /**
  * {@link CharsetActivator} - Activator for com.openexchange.charset bundle
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class CharsetActivator implements BundleActivator, ServiceTrackerCustomizer {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(CharsetActivator.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(CharsetActivator.class));
 
     private CollectionCharsetProvider collectionCharsetProvider;
 
-    private CharsetProvider backupCharsetProvider;
+    private CharsetProvider backupExtendedCharsetProvider;
+
+    private CharsetProvider backupStandardCharsetProvider;
 
     private BundleContext context;
 
@@ -82,10 +85,6 @@ public final class CharsetActivator implements BundleActivator, ServiceTrackerCu
         super();
     }
 
-    /*
-     * (non-Javadoc)
-     * @see org.osgi.util.tracker.ServiceTrackerCustomizer#addingService(org.osgi.framework.ServiceReference)
-     */
     public Object addingService(final ServiceReference reference) {
         final Object addedService = context.getService(reference);
         if (addedService instanceof CharsetProvider) {
@@ -97,17 +96,10 @@ public final class CharsetActivator implements BundleActivator, ServiceTrackerCu
         return addedService;
     }
 
-    /*
-     * (non-Javadoc)
-     * @see org.osgi.util.tracker.ServiceTrackerCustomizer#modifiedService(org.osgi.framework.ServiceReference, java.lang.Object)
-     */
     public void modifiedService(final ServiceReference reference, final Object service) {
+        // Nope
     }
 
-    /*
-     * (non-Javadoc)
-     * @see org.osgi.util.tracker.ServiceTrackerCustomizer#removedService(org.osgi.framework.ServiceReference, java.lang.Object)
-     */
     public void removedService(final ServiceReference reference, final Object service) {
         if (service instanceof CharsetProvider) {
             collectionCharsetProvider.removeCharsetProvider((CharsetProvider) service);
@@ -118,19 +110,21 @@ public final class CharsetActivator implements BundleActivator, ServiceTrackerCu
         context.ungetService(reference);
     }
 
-    /*
-     * (non-Javadoc)
-     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
-     */
     public void start(final BundleContext context) throws Exception {
         LOG.info("starting bundle: com.openexchange.charset");
 
         try {
-            final CharsetProvider[] results = ModifyCharsetExtendedProvider.modifyCharsetExtendedProvider();
-            backupCharsetProvider = results[0];
-            collectionCharsetProvider = (CollectionCharsetProvider) results[1];
+            {
+                final CharsetProvider[] results = ModifyCharsetExtendedProvider.modifyCharsetExtendedProvider();
+                backupExtendedCharsetProvider = results[0];
+                collectionCharsetProvider = (CollectionCharsetProvider) results[1];
+            }
+            {
+                final CharsetProvider[] results = ModifyCharsetStandardProvider.modifyCharsetExtendedProvider();
+                backupStandardCharsetProvider = null == results ? null : results[0];
+            }
             if (LOG.isInfoEnabled()) {
-                LOG.info("External charset provider replaced with collection charset provider");
+                LOG.info("Standard & external charset provider replaced with collection charset provider");
             }
             /*
              * Initialize a service tracker to track bundle chars providers
@@ -147,23 +141,24 @@ public final class CharsetActivator implements BundleActivator, ServiceTrackerCu
         }
     }
 
-    /*
-     * (non-Javadoc)
-     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
-     */
     public void stop(final BundleContext context) throws Exception {
         LOG.info("stopping bundle: com.openexchange.charset");
-
         try {
             serviceTracker.close();
             /*
              * Restore original
              */
-            ModifyCharsetExtendedProvider.restoreCharsetExtendedProvider(backupCharsetProvider);
-            backupCharsetProvider = null;
+            if (null != backupExtendedCharsetProvider) {
+                ModifyCharsetExtendedProvider.restoreCharsetExtendedProvider(backupExtendedCharsetProvider);
+                backupExtendedCharsetProvider = null;
+            }
+            if (null != backupStandardCharsetProvider) {
+                ModifyCharsetStandardProvider.restoreCharsetExtendedProvider(backupStandardCharsetProvider);
+                backupStandardCharsetProvider = null;
+            }
             collectionCharsetProvider = null;
             if (LOG.isInfoEnabled()) {
-                LOG.info("Collection charset provider replaced with former external charset provider");
+                LOG.info("Collection charset provider replaced with former standard/external charset provider");
                 LOG.info("Charset bundle successfully stopped");
             }
         } catch (final Throwable t) {
